//
//  Life.java
//  Life
//
//  Created by Jason on 21/09/2008.
//  Copyright (c) 2008 __MyCompanyName__. All rights reserved.
//
import java.util.*;
import java.awt.image.*;
import java.awt.*;
import javax.swing.*;
import javax.swing.event.ChangeListener;
import javax.swing.event.ChangeEvent;

public class Life {
    int currentBoard[][], nextBoard[][];
    int width, height;
    JFrame view;
	
    public Life(int w, int h) {
		width = w;
		height = h;
		/*
		 Notice that, in the array, the Y-coord comes before the
		 X-coord...  This is simply due to how Java (and every other)
		 programming language handles multidimensional arrays.  We
		 use the functions getState(int x, int y) and setState(int x, int y, int state)
		 to hide this weird indexing.
		 */
		currentBoard = new int[height][width];
		nextBoard = new int[height][width];
    }
	
    // This sets the state for the whole board.
    public void setState(int board[][]) {
		currentBoard = board;
    }
    
    // This sets the state for a particular cell in the next board.
    public void setNextState(int x, int y, int value) {
		nextBoard[y][x] = value;
    }
	
    // This returns the whole board as a two-dimensional array.
    public int[][] getState() {
		return currentBoard;
    }
	
    // This returns the value of a particular cell.
    public int getState(int x, int y) {
		return currentBoard[y][x];
    }
    
    public void step() {
		
		for (int i=0; i<width; i++) {
			for (int j=0; j<height; j++) {
				int lft = (i-1+width) % width;
				int rgt = (i+1+width) % width;
				int top = (j-1+height) % height;
				int bot = (j+1+height) % height;
				
				int neighbors = getState(lft, top) + getState(i, top) + getState(rgt, top)
				+ getState(lft, j) + getState(rgt, j)
				+ getState(lft, bot) + getState(i, bot) + getState(rgt, bot);
				
				int currentState = getState(i, j);
				int nextState;
				
				switch (neighbors) {
					case 0:
					case 1: nextState = 0; break;
					case 2: nextState = (currentState == 1)? 1 : 0; break;
					case 3: nextState = 1; break;
					default: nextState = 0;
				}
				
				setNextState(i, j, nextState);
			}
		}
		
		// Swap the references for currentBoard and nextBoard,
		// avoiding constantly having to declare and free up memory.
		int tmp[][];
		tmp = currentBoard;
		currentBoard = nextBoard;
		nextBoard = tmp;
    }
}