(* Content-type: application/mathematica *)

(*** Wolfram Notebook File ***)
(* http://www.wolfram.com/nb *)

(* CreatedBy='Mathematica 6.0' *)

(*CacheID: 234*)
(* Internal cache information:
NotebookFileLineBreakTest
NotebookFileLineBreakTest
NotebookDataPosition[       145,          7]
NotebookDataLength[     19788,        540]
NotebookOptionsPosition[     18135,        484]
NotebookOutlinePosition[     18515,        501]
CellTagsIndexPosition[     18472,        498]
WindowFrame->Normal*)

(* Beginning of Notebook Content *)
Notebook[{

Cell[CellGroupData[{
Cell["The Game of Life", "Title",
 CellChangeTimes->{{3.430999029778797*^9, 3.430999031277062*^9}}],

Cell[TextData[{
 "Requires ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " version 6"
}], "Subtitle",
 CellChangeTimes->{{3.43100221856812*^9, 3.431002222783174*^9}}],

Cell[TextData[{
 "Dr J. McKenzie Alexander\[LineSeparator]",
 StyleBox["Department of Philosophy, Logic and Scientific Method\nLondon \
School of Economics",
  FontSize->14]
}], "Subsubtitle",
 CellChangeTimes->{{3.431002176725724*^9, 3.431002201693757*^9}}],

Cell[TextData[{
 "All ",
 StyleBox["J/Link",
  FontSlant->"Italic"],
 " models have to load the necessary ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " library and then \"install\" Java \[LongDash] i.e., provide the necessary \
hooks between the Java runtime engine and the current ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " session."
}], "Text",
 CellChangeTimes->{{3.431000138848542*^9, 3.431000216663294*^9}}],

Cell[BoxData[{
 RowBox[{
  RowBox[{"Needs", "[", "\"\<JLink`\>\"", "]"}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"InstallJava", "[", "]"}], ";"}]}], "Input",
 CellChangeTimes->{{3.430999036160818*^9, 3.430999045854837*^9}}],

Cell[TextData[{
 "This actually allows you to use all of the standard Java classes. However, \
to use the program code contained within Life.jar (the compiled code from \
Life.java) you need to add Life.jar to the current class path so that ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " can find it. Normally the argument to ",
 StyleBox["AddToClassPath[] ",
  FontWeight->"Bold"],
 "is a string containing the full path to the JAR file. However, because I've \
put Life.jar in the same directory as this notebook, you can use the \
following bit of code to construct the path to Life.jar (the current value \
shown is what I had on my computer):"
}], "Text",
 CellChangeTimes->{{3.431000220652445*^9, 3.431000374341165*^9}, {
  3.431001343184314*^9, 3.431001354287726*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"NotebookDirectory", "[", "]"}], "<>", 
  "\"\<Life.jar\>\""}]], "Input",
 CellChangeTimes->{3.4309991734029818`*^9}],

Cell[BoxData["\<\"/Users/jalex/Dropbox/Mathematica/Life/Life.jar\"\>"], \
"Output",
 CellChangeTimes->{{3.430999169919055*^9, 3.4309991737842607`*^9}, 
   3.431000375837358*^9, 3.431002255573408*^9, 3.431002300142477*^9, 
   3.5158470006513577`*^9, 3.516115187213826*^9, 3.522152674228281*^9}]
}, Open  ]],

Cell["So now add that path to the current class path:", "Text",
 CellChangeTimes->{{3.4310003827109632`*^9, 3.431000396389421*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"AddToClassPath", "[", 
   RowBox[{
    RowBox[{"NotebookDirectory", "[", "]"}], "<>", "\"\</Life.jar\>\""}], 
   "]"}], ";"}]], "Input",
 CellChangeTimes->{{3.430999048323255*^9, 3.430999089335924*^9}, {
  3.43099915686784*^9, 3.4309991914291*^9}}],

Cell[TextData[{
 "Having done that, you are now ready to construct a Java object for the game \
of Life. The function ",
 StyleBox["JavaNew[]",
  FontWeight->"Bold"],
 " will create a new object and pass the remaining arguments to the \
constructor for that object.  ",
 StyleBox["JavaNew[]",
  FontWeight->"Bold"],
 " returns a reference to that Java object which you can store in a ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " variable."
}], "Text",
 CellChangeTimes->{{3.431000400040681*^9, 3.431000478704588*^9}}],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{"width", "=", "500"}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"height", "=", "500"}], ";"}], "\[IndentingNewLine]", 
 RowBox[{"m", "=", 
  RowBox[{"JavaNew", "[", 
   RowBox[{"\"\<Life\>\"", ",", "width", ",", "height"}], "]"}]}]}], "Input",
 CellChangeTimes->{{3.4309991949042187`*^9, 3.430999204171957*^9}, {
  3.430999254546674*^9, 3.430999255449761*^9}, {3.4309996092310038`*^9, 
  3.43099963996342*^9}, {3.5160977095921164`*^9, 3.516097710893943*^9}, {
  3.5160978329437523`*^9, 3.516097843215477*^9}}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\[LeftGuillemet]", 
   RowBox[{"JavaObject", "[", "Life", "]"}], "\[RightGuillemet]"}],
  JLink`Objects`vm1`JavaObject34254567973584897]], "Output",
 CellChangeTimes->{
  3.4309992055245123`*^9, {3.430999256426107*^9, 3.430999265118371*^9}, 
   3.4309993124355392`*^9, 3.430999612435354*^9, 3.430999657666184*^9, 
   3.430999698661129*^9, 3.431000480100404*^9, 3.431002310046176*^9, 
   3.515847008272768*^9, 3.516097676705386*^9, 3.5160977114722013`*^9, 
   3.5160978437235107`*^9, {3.516115178518825*^9, 3.516115191942058*^9}, 
   3.52215268061943*^9}]
}, Open  ]],

Cell[TextData[{
 "This next bit of code does two things.  It begins by calling the method \
\"setState(...)\" for the object stored in ",
 StyleBox["m",
  FontWeight->"Bold"],
 ". Notice that there is a slight difference from ordinary Java syntax.  In a \
normal Java program, one would typically write:\n\n\tm.setState( ..... )\n\t\n\
However, when calling Java functions from ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 ",  a slightly different syntax is required. This is requried for two \
reasons. First, \".\" is interpreted by ",
 StyleBox["Mathematica ",
  FontSlant->"Italic"],
 "to be the  decimal point. Second, ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " uses the parenthesis for grouping and square brackets to indicate the \
arguments to a function. What you have to keep in mind is that you use \"@\" \
to invoke a function for a Java object and square brackets to surround the \
arguments to the function."
}], "Text",
 CellChangeTimes->{{3.431000494945084*^9, 3.4310005726837606`*^9}, {
  3.431000638744273*^9, 3.431000920888237*^9}}],

Cell[TextData[{
 "The second thing the next bit of code does is to store the current state of \
the model in the variable ",
 StyleBox["currentState",
  FontWeight->"Bold"],
 ". The reason we do this will be explained monentarily."
}], "Text",
 CellChangeTimes->{{3.4310009243269176`*^9, 3.43100095504145*^9}}],

Cell[BoxData[{
 RowBox[{
  RowBox[{"m", "@", 
   RowBox[{"setState", "[", "\[IndentingNewLine]", 
    RowBox[{"Table", "[", 
     RowBox[{
      RowBox[{"RandomInteger", "[", 
       RowBox[{
        RowBox[{"{", 
         RowBox[{"0", ",", "1"}], "}"}], ",", 
        RowBox[{"{", "width", "}"}]}], "]"}], ",", 
      RowBox[{"{", "height", "}"}]}], "]"}], "\[IndentingNewLine]", "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"currentState", "=", 
   RowBox[{"m", "@", 
    RowBox[{"getState", "[", "]"}]}]}], ";"}]}], "Input",
 CellChangeTimes->{{3.43099931347901*^9, 3.430999348841342*^9}, {
  3.430999426190261*^9, 3.430999427674272*^9}, {3.430999460471224*^9, 
  3.430999464437395*^9}, {3.430999589970333*^9, 3.430999645308221*^9}, {
  3.516097714511078*^9, 3.5160977164864807`*^9}, {3.516097846915353*^9, 
  3.516097849752901*^9}}],

Cell[TextData[{
 "Having evaluated the above, you should have seen a graphic image appear \
below. This happened because the following ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " code declares the output of the ",
 StyleBox["Graphics",
  FontWeight->"Bold"],
 " command to be ",
 StyleBox["Dynamic",
  FontWeight->"Bold"],
 ". That means that ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " tracks the dependencies for the Graphics command and re-evaluates it \
(updating the display) every time any of the variables on which it depends \
change. (Check that this is true by re-evaluating the above cell a couple of \
times. The image below should refresh.)"
}], "Text",
 CellChangeTimes->{{3.431000968260326*^9, 3.4310010963183727`*^9}}],

Cell[TextData[{
 "As for what the meaning of the code below is, it's just using the standard ",
 StyleBox["Raster",
  FontWeight->"Bold"],
 " command from ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 ". The first argument is the array of integers to draw in the raster, the \
second argument is the subregion of the array to draw (setting it to ",
 StyleBox["Automatic ",
  FontWeight->"Bold"],
 "causes the whole array to be drawn), and the third argument allows us to \
adjust the color scheme.  What I've done below by giving ",
 StyleBox["{1, -.25}",
  FontWeight->"Bold"],
 " as the third argument is to invert the standard color scheme (i.e., 0 = \
black, 1 = white) so that cells which are alive (i.e., have a value 1) are \
drawn black, and to draw cells which are dead (i.e., have a value 0) by a \
light gray so that the full dimensions of the array are visible."
}], "Text",
 CellChangeTimes->{{3.43100110508479*^9, 3.43100132133642*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Dynamic", "@", 
  RowBox[{"Graphics", "[", 
   RowBox[{
    RowBox[{"{", "\[IndentingNewLine]", 
     RowBox[{"Raster", "[", 
      RowBox[{"currentState", ",", "Automatic", ",", 
       RowBox[{"{", 
        RowBox[{"1", ",", 
         RowBox[{"-", ".25"}]}], "}"}]}], "]"}], "\[IndentingNewLine]", "}"}],
     ",", 
    RowBox[{"ImageSize", "\[Rule]", "700"}]}], "]"}]}]], "Input",
 CellChangeTimes->{{3.4309992677954397`*^9, 3.430999300835884*^9}, {
  3.4309995328958807`*^9, 3.430999543008263*^9}, {3.430999766448413*^9, 
  3.43099988202024*^9}, {3.4310012994734*^9, 3.431001306263014*^9}, {
  3.5160977342289877`*^9, 3.516097761075708*^9}}],

Cell[BoxData[
 DynamicBox[ToBoxes[
   Graphics[{
     Raster[$CellContext`currentState, Automatic, {1, -0.25}]}, ImageSize -> 
    700], StandardForm],
  ImageSizeCache->{875., {434., 440.}}]], "Output",
 CellChangeTimes->{
  3.430999301240443*^9, 3.43099935243295*^9, 3.430999429106938*^9, {
   3.4309994658105583`*^9, 3.430999469476265*^9}, {3.430999535407894*^9, 
   3.430999543641919*^9}, {3.43099978907691*^9, 3.430999882526102*^9}, {
   3.431001301953178*^9, 3.431001306869028*^9}, 3.515847018282243*^9, {
   3.516097738875925*^9, 3.5160977618390417`*^9}, 3.516115197746307*^9, 
   3.522152689172573*^9}]
}, Open  ]],

Cell[TextData[{
 "In order to step the game of life forward, simply call the function ",
 StyleBox["step()",
  FontWeight->"Bold"],
 ". You need to update the value of ",
 StyleBox["currentState",
  FontWeight->"Bold"],
 " in order to have the above graphic refresh."
}], "Text",
 CellChangeTimes->{{3.43100136970228*^9, 3.431001417913932*^9}}],

Cell[BoxData[{
 RowBox[{
  RowBox[{"m", "@", 
   RowBox[{"step", "[", "]"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"currentState", "=", 
   RowBox[{"m", "@", 
    RowBox[{"getState", "[", "]"}]}]}], ";"}]}], "Input",
 CellChangeTimes->{{3.430999479655757*^9, 3.430999482277831*^9}, {
  3.430999545648326*^9, 3.4309995554140987`*^9}}],

Cell[TextData[{
 "If you want to step the model forward a number of times, you can wrap the \
above code in a ",
 StyleBox["Do-",
  FontWeight->"Bold"],
 "loop. Notice that the code below contains the function ",
 StyleBox["FinishDynamic[]",
  FontWeight->"Bold"],
 ". This instructs ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " to finish updating any Dynamic objects which are displayed before \
continuing to evaluate the expression. (If you remove that expression, you \
won't see every interation displayed... the Java code runs so much faster \
than the ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " display code that the front end can't keep up. Try deleting \
FinishDynamic[] and see!)"
}], "Text",
 CellChangeTimes->{{3.431001423716713*^9, 3.4310015757834663`*^9}}],

Cell[BoxData[
 RowBox[{"Do", "[", "\[IndentingNewLine]", 
  RowBox[{
   RowBox[{
    RowBox[{"m", "@", 
     RowBox[{"step", "[", "]"}]}], ";", "\[IndentingNewLine]", 
    RowBox[{"currentState", "=", 
     RowBox[{"m", "@", 
      RowBox[{"getState", "[", "]"}]}]}], ";", "\[IndentingNewLine]", 
    RowBox[{"FinishDynamic", "[", "]"}]}], ",", 
   RowBox[{"{", "100", "}"}]}], "\[IndentingNewLine]", "]"}]], "Input",
 CellChangeTimes->{{3.430999479655757*^9, 3.430999482277831*^9}, {
  3.430999545648326*^9, 3.430999576765774*^9}, {3.4310015116959677`*^9, 
  3.4310015234712543`*^9}, {3.431001580106098*^9, 3.43100158352168*^9}}],

Cell[TextData[{
 "Ok. Now lets create a row of buttons to control the model. The ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " command ",
 StyleBox["Button[]",
  FontWeight->"Bold"],
 ", in its simplest form, takes two arguments: the label, and the function to \
evaluate when the button is pressed. (There's more that can be said, but \
that's the basic idea.) Here's an example:"
}], "Text",
 CellChangeTimes->{{3.4310016053253183`*^9, 3.4310016717468653`*^9}, 
   3.431001795088929*^9}],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{"x", "=", "0"}], ";"}], "\[IndentingNewLine]", 
 RowBox[{"Button", "[", 
  RowBox[{"\"\<Add 10 to x\>\"", ",", 
   RowBox[{"Function", "[", 
    RowBox[{"Print", "[", 
     RowBox[{"x", "+=", "10"}], "]"}], "]"}]}], "]"}]}], "Input",
 CellChangeTimes->{{3.4310016731101522`*^9, 3.431001706829249*^9}, {
  3.4310017503073874`*^9, 3.431001762831088*^9}}],

Cell[BoxData[
 ButtonBox["\<\"Add 10 to x\"\>",
  Appearance->Automatic,
  ButtonFunction:>(Print[
    AddTo[$CellContext`x, 10]]& ),
  Evaluator->Automatic,
  Method->"Preemptive"]], "Output",
 CellChangeTimes->{{3.43100168412112*^9, 3.431001708311028*^9}, 
   3.4310017643243837`*^9, 3.431002356921854*^9, 3.522152793491724*^9}]
}, Open  ]],

Cell[TextData[{
 "The ",
 StyleBox["Row[{...}]",
  FontWeight->"Bold"],
 " command allows us to put a bunch of buttons in a single row. (There's also \
a function called ",
 StyleBox["Column[{...}].)",
  FontWeight->"Bold"],
 " Each of the buttons below has a function which does the same thing as the \
input cells you've seen above.  The only slight difference is for the third \
button, which has the option ",
 StyleBox["Method->\"Queued\" ",
  FontWeight->"Bold"],
 "specified. Normally a button is pre-emptive, which means that it grabs \
control of the front end and won't let the front end respond to anything \
(like a window resize or a scroll) until the button finishes computing its \
function. For buttons which take a while to finish what they are trying to \
do, that can be a bit annoying. Setting a button's method to be \"Queued\" \
means that the button is willing to relinquish control in mid-computation to \
allow the front end to do things like scroll and resize windows."
}], "Text",
 CellChangeTimes->{{3.4310018020550833`*^9, 3.431002018423193*^9}}],

Cell[TextData[{
 "Evaluate the cell below and click on the \"Reset model\" button. Scroll up \
to verify that the model has changed its state. Then scroll down and click on \
the \"Step 100 iterations\" button and scroll up to verify the model is \
running and updating the display in ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 "."
}], "Text",
 CellChangeTimes->{{3.431002035310343*^9, 3.4310020802581387`*^9}}],

Cell["\<\
If you want, you can copy the output cell containing the button bar and paste \
it just below the graphic displaying the state of the model. Try it!\
\>", "Text",
 CellChangeTimes->{{3.431002083226993*^9, 3.4310021157149487`*^9}}],

Cell["\<\
And, just for the record, this just barely scratches the surface of what is \
possible...\
\>", "Text",
 CellChangeTimes->{{3.4310021430072403`*^9, 3.4310021639885187`*^9}}],

Cell[BoxData[
 RowBox[{"Row", "[", 
  RowBox[{"{", "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{"Button", "[", 
     RowBox[{"\"\<Reset model\>\"", ",", "\[IndentingNewLine]", 
      RowBox[{"Function", "[", "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"m", "@", 
         RowBox[{"setState", "[", 
          RowBox[{"Table", "[", 
           RowBox[{
            RowBox[{"RandomInteger", "[", 
             RowBox[{
              RowBox[{"{", 
               RowBox[{"0", ",", "1"}], "}"}], ",", 
              RowBox[{"{", "500", "}"}]}], "]"}], ",", 
            RowBox[{"{", "500", "}"}]}], "]"}], "]"}]}], ";", 
        "\[IndentingNewLine]", 
        RowBox[{"currentState", "=", 
         RowBox[{"m", "@", 
          RowBox[{"getState", "[", "]"}]}]}], ";"}], "\[IndentingNewLine]", 
       "]"}]}], "\[IndentingNewLine]", "]"}], ",", "\[IndentingNewLine]", 
    RowBox[{"Button", "[", 
     RowBox[{"\"\<Step one iteration\>\"", ",", "\[IndentingNewLine]", 
      RowBox[{"Function", "[", "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"m", "@", 
         RowBox[{"step", "[", "]"}]}], ";", "\[IndentingNewLine]", 
        RowBox[{"currentState", "=", 
         RowBox[{"m", "@", 
          RowBox[{"getState", "[", "]"}]}]}], ";"}], "\[IndentingNewLine]", 
       "]"}]}], "\[IndentingNewLine]", "]"}], ",", "\[IndentingNewLine]", 
    RowBox[{"Button", "[", 
     RowBox[{"\"\<Step 100 iterations\>\"", ",", "\[IndentingNewLine]", 
      RowBox[{"Function", "[", "\[IndentingNewLine]", 
       RowBox[{"Do", "[", "\[IndentingNewLine]", 
        RowBox[{
         RowBox[{
          RowBox[{"m", "@", 
           RowBox[{"step", "[", "]"}]}], ";", "\[IndentingNewLine]", 
          RowBox[{"currentState", "=", 
           RowBox[{"m", "@", 
            RowBox[{"getState", "[", "]"}]}]}], ";", "\[IndentingNewLine]", 
          RowBox[{"FinishDynamic", "[", "]"}]}], ",", 
         RowBox[{"{", "100", "}"}]}], "\[IndentingNewLine]", "]"}], 
       "\[IndentingNewLine]", "]"}], ",", 
      RowBox[{"Method", "\[Rule]", "\"\<Queued\>\""}]}], 
     "\[IndentingNewLine]", "]"}]}], "\[IndentingNewLine]", "}"}], 
  "]"}]], "Input",
 CellChangeTimes->{{3.4309997199063683`*^9, 3.430999731750663*^9}, {
  3.430999914368185*^9, 3.431000074821862*^9}, {3.522152746034911*^9, 
  3.522152751144311*^9}}]
}, Open  ]]
},
WindowSize->{1216, 1046},
WindowMargins->{{144, Automatic}, {Automatic, 0}},
ShowSelection->True,
FrontEndVersion->"8.0 for Mac OS X x86 (32-bit, 64-bit Kernel) (February 23, \
2011)",
StyleDefinitions->"Default.nb"
]
(* End of Notebook Content *)

(* Internal cache information *)
(*CellTagsOutline
CellTagsIndex->{}
*)
(*CellTagsIndex
CellTagsIndex->{}
*)
(*NotebookFileOutline
Notebook[{
Cell[CellGroupData[{
Cell[567, 22, 99, 1, 95, "Title"],
Cell[669, 25, 173, 6, 57, "Subtitle"],
Cell[845, 33, 258, 6, 69, "Subsubtitle"],
Cell[1106, 41, 433, 13, 51, "Text"],
Cell[1542, 56, 235, 5, 53, "Input"],
Cell[1780, 63, 783, 15, 70, "Text"],
Cell[CellGroupData[{
Cell[2588, 82, 151, 4, 33, "Input"],
Cell[2742, 88, 293, 4, 33, "Output"]
}, Open  ]],
Cell[3050, 95, 131, 1, 32, "Text"],
Cell[3184, 98, 283, 7, 33, "Input"],
Cell[3470, 107, 527, 14, 51, "Text"],
Cell[CellGroupData[{
Cell[4022, 125, 554, 11, 72, "Input"],
Cell[4579, 138, 599, 11, 33, "Output"]
}, Open  ]],
Cell[5193, 152, 1071, 23, 164, "Text"],
Cell[6267, 177, 310, 7, 32, "Text"],
Cell[6580, 186, 854, 21, 91, "Input"],
Cell[7437, 209, 756, 19, 70, "Text"],
Cell[8196, 230, 957, 20, 89, "Text"],
Cell[CellGroupData[{
Cell[9178, 254, 669, 15, 72, "Input"],
Cell[9850, 271, 610, 12, 892, "Output"]
}, Open  ]],
Cell[10475, 286, 344, 9, 32, "Text"],
Cell[10822, 297, 348, 9, 53, "Input"],
Cell[11173, 308, 792, 20, 70, "Text"],
Cell[11968, 330, 630, 13, 110, "Input"],
Cell[12601, 345, 498, 12, 51, "Text"],
Cell[CellGroupData[{
Cell[13124, 361, 387, 9, 53, "Input"],
Cell[13514, 372, 330, 8, 51, "Output"]
}, Open  ]],
Cell[13859, 383, 1075, 21, 107, "Text"],
Cell[14937, 406, 421, 9, 51, "Text"],
Cell[15361, 417, 240, 4, 32, "Text"],
Cell[15604, 423, 183, 4, 32, "Text"],
Cell[15790, 429, 2329, 52, 452, "Input"]
}, Open  ]]
}
]
*)

(* End of internal cache information *)
